@@ -89,7 +89,7 @@ class GetItemNode(Node):
     ----------
     value : Any
         The value from which to get the item.
-    key : int or slice or List[int or slice]
+    key : int or slice or list[int or slice]
         The key for the item or items.
     name : str, optional
         The name of the node.
