@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -14,22 +14,22 @@
 # pylint: disable=too-many-lines
 """Module for binary operation nodes."""
 
-import warnings
 from typing import Union
 
 import forge
 import numpy as np
 
 from qctrlcommons.exceptions import QctrlException
-from qctrlcommons.node import (
-    node_data,
-    types,
-)
 from qctrlcommons.node.base import Node
+from qctrlcommons.node.node_data import (
+    Pwc,
+    Stf,
+    Tensor,
+)
 from qctrlcommons.node.utils import (
     NumericOrFunction,
     is_broadcastable,
-    validate_batch_and_values_shapes,
+    validate_batch_and_value_shapes,
     validate_broadcasted_shape,
     validate_function_output_shapes,
     validate_shape,
@@ -47,7 +47,7 @@ def _create_flexible_binary_node_data(
     x,
     y,
     name,
-    validate_values_shape=validate_broadcasted_shape,
+    validate_value_shape=validate_broadcasted_shape,
 ):
     """
     Common implementation of `create_node_data` for nodes acting on Tensors, Pwcs, and Stfs
@@ -59,80 +59,70 @@ def _create_flexible_binary_node_data(
         The operation to implement.
     op_name : str
         The name of the operation.
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The left operand.
-    y : number or np.ndarray or Tensor or TensorPwc or Stf
+    y : number or np.ndarray or Tensor or Pwc or Stf
         The right operand.
     name : str
         The name of the node.
-    validate_values_shape : Callable[[tuple, tuple, str, str], tuple], optional
-        Function that takes the values shapes of two Tensors, TensorPwcs,
+    validate_value_shape : Callable[[tuple, tuple, str, str], tuple], optional
+        Function that takes the value shapes of two Tensors, Pwcs,
         or Stfs (as well as their names), and returns the expected values
-        shape of the output Tensor, TensorPwc, or Stf. The function
+        shape of the output Tensor, Pwc, or Stf. The function
         shouldn't assume that the shapes are compatible, and raise an
         exception if they aren't. The names provided should be used to
         generate the error message.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The operation acting on the object.
     """
     # operation(Pwc, Stf) or operation(Stf, Pwc)
     check_argument(
-        not (
-            isinstance(x, node_data.TensorPwcNodeData)
-            and isinstance(y, node_data.StfNodeData)
-        )
-        and not (
-            isinstance(x, node_data.StfNodeData)
-            and isinstance(y, node_data.TensorPwcNodeData)
-        ),
-        f"You can't apply the {op_name} operation between a TensorPwc and an Stf.",
+        not (isinstance(x, Pwc) and isinstance(y, Stf))
+        and not (isinstance(x, Stf) and isinstance(y, Pwc)),
+        f"You can't apply the {op_name} operation between a Pwc and an Stf.",
         {"x": x, "y": y},
     )
 
     # operation(Stf, Stf)
-    if isinstance(x, node_data.StfNodeData) and isinstance(y, node_data.StfNodeData):
+    if isinstance(x, Stf) and isinstance(y, Stf):
 
         check_argument(
             name is None, "You can't assign a name to an Stf node.", {"name": name}
         )
 
-        x_batch_shape, x_values_shape = validate_batch_and_values_shapes(x, "x")
-        y_batch_shape, y_values_shape = validate_batch_and_values_shapes(y, "y")
+        x_batch_shape, x_value_shape = validate_batch_and_value_shapes(x, "x")
+        y_batch_shape, y_value_shape = validate_batch_and_value_shapes(y, "y")
 
-        batch_shape, values_shape = validate_function_output_shapes(
+        batch_shape, value_shape = validate_function_output_shapes(
             x_batch_shape,
-            x_values_shape,
+            x_value_shape,
             y_batch_shape,
-            y_values_shape,
-            validate_values_shape=validate_values_shape,
-        )
-        return node_data.StfNodeData(
-            _operation, values_shape=values_shape, batch_shape=batch_shape
+            y_value_shape,
+            validate_value_shape=validate_value_shape,
         )
+        return Stf(_operation, value_shape=value_shape, batch_shape=batch_shape)
 
     # operation(Pwc, Pwc)
-    if isinstance(x, node_data.TensorPwcNodeData) and isinstance(
-        y, node_data.TensorPwcNodeData
-    ):
+    if isinstance(x, Pwc) and isinstance(y, Pwc):
 
         check_argument(
             np.isclose(np.sum(x.durations), np.sum(y.durations)),
-            "Both TensorPwc terms must have the same total duration.",
+            "Both Pwc terms must have the same total duration.",
             {"x": x, "y": y},
         )
 
-        x_batch_shape, x_values_shape = validate_batch_and_values_shapes(x, "x")
-        y_batch_shape, y_values_shape = validate_batch_and_values_shapes(y, "y")
+        x_batch_shape, x_value_shape = validate_batch_and_value_shapes(x, "x")
+        y_batch_shape, y_value_shape = validate_batch_and_value_shapes(y, "y")
 
-        batch_shape, values_shape = validate_function_output_shapes(
+        batch_shape, value_shape = validate_function_output_shapes(
             x_batch_shape,
-            x_values_shape,
+            x_value_shape,
             y_batch_shape,
-            y_values_shape,
-            validate_values_shape=validate_values_shape,
+            y_value_shape,
+            validate_value_shape=validate_value_shape,
         )
 
         upsampled_durations = np.diff(
@@ -145,22 +135,22 @@ def _create_flexible_binary_node_data(
             )
         )
 
-        return node_data.TensorPwcNodeData(
+        return Pwc(
             _operation,
-            values_shape=values_shape,
+            value_shape=value_shape,
             durations=upsampled_durations,
             batch_shape=batch_shape,
         )
 
     # operation(Stf, Tensor) or operation(Tensor, Stf)
-    if isinstance(x, node_data.StfNodeData) or isinstance(y, node_data.StfNodeData):
+    if isinstance(x, Stf) or isinstance(y, Stf):
 
         check_argument(
             name is None, "You can't assign a name to an Stf node.", {"name": name}
         )
 
-        if isinstance(x, node_data.StfNodeData):
-            f_batch_shape, f_values_shape = validate_batch_and_values_shapes(x, "x")
+        if isinstance(x, Stf):
+            f_batch_shape, f_value_shape = validate_batch_and_value_shapes(x, "x")
             f_name = "x"
             check_argument_numeric(y, "y")
             t_shape = validate_shape(y, "y")
@@ -170,31 +160,27 @@ def _create_flexible_binary_node_data(
             check_argument_numeric(x, "x")
             t_shape = validate_shape(x, "x")
             t_name = "x"
-            f_batch_shape, f_values_shape = validate_batch_and_values_shapes(y, "y")
+            f_batch_shape, f_value_shape = validate_batch_and_value_shapes(y, "y")
             f_name = "y"
             tensor_first = True
 
-        batch_shape, values_shape = validate_tensor_and_function_output_shapes(
+        batch_shape, value_shape = validate_tensor_and_function_output_shapes(
             t_shape,
             f_batch_shape,
-            f_values_shape,
+            f_value_shape,
             t_name,
             f_name,
-            validate_values_shape=validate_values_shape,
+            validate_value_shape=validate_value_shape,
             tensor_first=tensor_first,
         )
 
-        return node_data.StfNodeData(
-            _operation, values_shape=values_shape, batch_shape=batch_shape
-        )
+        return Stf(_operation, value_shape=value_shape, batch_shape=batch_shape)
 
     # operation(Pwc, Tensor) or operation(Tensor, Pwc)
-    if isinstance(x, node_data.TensorPwcNodeData) or isinstance(
-        y, node_data.TensorPwcNodeData
-    ):
+    if isinstance(x, Pwc) or isinstance(y, Pwc):
 
-        if isinstance(x, node_data.TensorPwcNodeData):
-            f_batch_shape, f_values_shape = validate_batch_and_values_shapes(x, "x")
+        if isinstance(x, Pwc):
+            f_batch_shape, f_value_shape = validate_batch_and_value_shapes(x, "x")
             f_name = "x"
             durations = x.durations
             check_argument_numeric(y, "y")
@@ -205,24 +191,24 @@ def _create_flexible_binary_node_data(
             check_argument_numeric(x, "x")
             t_shape = validate_shape(x, "x")
             t_name = "x"
-            f_batch_shape, f_values_shape = validate_batch_and_values_shapes(y, "y")
+            f_batch_shape, f_value_shape = validate_batch_and_value_shapes(y, "y")
             f_name = "y"
             durations = y.durations
             tensor_first = True
 
-        batch_shape, values_shape = validate_tensor_and_function_output_shapes(
+        batch_shape, value_shape = validate_tensor_and_function_output_shapes(
             t_shape,
             f_batch_shape,
-            f_values_shape,
+            f_value_shape,
             t_name,
             f_name,
-            validate_values_shape=validate_values_shape,
+            validate_value_shape=validate_value_shape,
             tensor_first=tensor_first,
         )
 
-        return node_data.TensorPwcNodeData(
+        return Pwc(
             _operation,
-            values_shape=values_shape,
+            value_shape=value_shape,
             durations=durations,
             batch_shape=batch_shape,
         )
@@ -232,34 +218,34 @@ def _create_flexible_binary_node_data(
     check_argument_numeric(y, "y")
     x_shape = validate_shape(x, "x")
     y_shape = validate_shape(y, "y")
-    shape = validate_values_shape(x_shape, y_shape, "x", "y")
-    return node_data.TensorNodeData(_operation, shape=shape)
+    shape = validate_value_shape(x_shape, y_shape, "x", "y")
+    return Tensor(_operation, shape=shape)
 
 
 class Addition(Node):
     """
-    Calculates the element-wise sum between numbers, np.ndarrays, Tensors, TensorPwcs,
+    Calculates the element-wise sum between numbers, np.ndarrays, Tensors, Pwcs,
     or Stfs. You can also use the arithmetic operator ``+`` to calculate their sum.
 
     Considering numbers and np.ndarrays as Tensors, if the two objects are of the same type,
-    so is the returned object. If the objects have different types, TensorPwcs and Stfs can operate
-    with a tensor (returning a TensorPwc or Stf, respectively).
+    so is the returned object. If the objects have different types, Pwcs and Stfs can operate
+    with a tensor (returning a Pwc or Stf, respectively).
 
     This operation supports broadcasting between the different objects. When operating a tensor-like
-    object with an Stf or TensorPwc, the time dimension of the latter is ignored.
+    object with an Stf or a Pwc, the time dimension of the latter is ignored.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The left summand, :math:`x`.
-    y : number or np.ndarray or Tensor or TensorPwc or Stf
+    y : number or np.ndarray or Tensor or Pwc or Stf
         The right summand, :math:`y`.
     name : str, optional
         The name of the node. You can only provide a name if neither `x` nor `y` are Stfs.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise sum :math:`x+y`.
     """
 
@@ -268,7 +254,7 @@ class Addition(Node):
         forge.arg("x", type=NumericOrFunction),
         forge.arg("y", type=NumericOrFunction),
     ]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -277,28 +263,28 @@ class Addition(Node):
 
 class Subtraction(Node):
     """
-    Calculates the element-wise difference between numbers, np.ndarrays, Tensors, TensorPwcs,
+    Calculates the element-wise difference between numbers, np.ndarrays, Tensors, Pwcs,
     or Stfs. You can also use the arithmetic operator ``-`` to calculate their difference.
 
     Considering numbers and np.ndarrays as Tensors, if the two objects are of the same type,
-    so is the returned object. If the objects have different types, TensorPwcs and Stfs can operate
-    with a tensor (returning a TensorPwc or Stf, respectively).
+    so is the returned object. If the objects have different types, Pwcs and Stfs can operate
+    with a tensor (returning a Pwc or Stf, respectively).
 
     This operation supports broadcasting between the different objects. When operating a tensor-like
-    object with an Stf or TensorPwc, the time dimension of the latter is ignored.
+    object with an Stf or a Pwc, the time dimension of the latter is ignored.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The minuend, :math:`x`.
-    y : number or np.ndarray or Tensor or TensorPwc or Stf
+    y : number or np.ndarray or Tensor or Pwc or Stf
         The subtrahend, :math:`y`.
     name : str, optional
         The name of the node. You can only provide a name if neither `x` nor `y` are Stfs.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise difference :math:`x-y`.
     """
 
@@ -307,54 +293,37 @@ class Subtraction(Node):
         forge.arg("x", type=NumericOrFunction),
         forge.arg("y", type=NumericOrFunction),
     ]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         return _create_flexible_binary_node_data(_operation, "subtract", **kwargs)
 
 
-class SubtractionDeprecated(Subtraction):
-    r"""
-    This node has been deprecated and will be removed in the future.
-    Please use :py:func:`subtract` instead.
-    """
-
-    name = "sub"
-
-    @classmethod
-    def create_node_data(cls, _operation, **kwargs):
-        warnings.warn(
-            "The 'sub' node will be removed in the future. "
-            "Please use 'subtract' instead."
-        )
-        return super().create_node_data(_operation, **kwargs)
-
-
 class Multiplication(Node):
     r"""
     Calculates the element-wise product between numbers, np.ndarrays, Tensors,
-    TensorPwcs, or Stfs. You can also use the arithmetic operator ``*`` to calculate their product.
+    Pwcs, or Stfs. You can also use the arithmetic operator ``*`` to calculate their product.
 
     Considering numbers and np.ndarrays as Tensors, if the two objects are of the same type,
-    so is the returned object. If the objects have different types, TensorPwcs and Stfs can operate
-    with a tensor (returning a TensorPwc or Stf, respectively).
+    so is the returned object. If the objects have different types, Pwcs and Stfs can operate
+    with a tensor (returning a Pwc or Stf, respectively).
 
     This operation supports broadcasting between the different objects. When operating a tensor-like
-    object with an Stf or TensorPwc, the time dimension of the latter is ignored.
+    object with an Stf or a Pwc, the time dimension of the latter is ignored.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The left factor, :math:`x`.
-    y : number or np.ndarray or Tensor or TensorPwc or Stf
+    y : number or np.ndarray or Tensor or Pwc or Stf
         The right factor, :math:`y`.
     name : str, optional
         The name of the node. You can only provide a name if neither `x` nor `y` are Stfs.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise product :math:`x \times y`.
     """
 
@@ -363,54 +332,37 @@ class Multiplication(Node):
         forge.arg("x", type=NumericOrFunction),
         forge.arg("y", type=NumericOrFunction),
     ]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         return _create_flexible_binary_node_data(_operation, "multiply", **kwargs)
 
 
-class MultiplicationDeprecated(Multiplication):
-    r"""
-    This node has been deprecated and will be removed in the future.
-    Please use :py:func:`multiply` instead.
-    """
-
-    name = "mul"
-
-    @classmethod
-    def create_node_data(cls, _operation, **kwargs):
-        warnings.warn(
-            "The 'mul' node will be removed in the future. "
-            "Please use 'multiply' instead."
-        )
-        return super().create_node_data(_operation, **kwargs)
-
-
 class TrueDivision(Node):
     """
-    Calculates the element-wise division between numbers, np.ndarrays, Tensors, TensorPwcs,
+    Calculates the element-wise division between numbers, np.ndarrays, Tensors, Pwcs,
     or Stfs. You can also use the arithmetic operator ``/`` to calculate their division.
 
     Considering numbers and np.ndarrays as Tensors, if the two objects are of the same type,
-    so is the returned object. If the objects have different types, TensorPwcs and Stfs can operate
-    with a tensor (returning a TensorPwc or Stf, respectively).
+    so is the returned object. If the objects have different types, Pwcs and Stfs can operate
+    with a tensor (returning a Pwc or Stf, respectively).
 
     This operation supports broadcasting between the different objects. When operating a tensor-like
-    object with an Stf or TensorPwc, the time dimension of the latter is ignored.
+    object with an Stf or a Pwc, the time dimension of the latter is ignored.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The numerator, :math:`x`.
-    y : number or np.ndarray or Tensor or TensorPwc or Stf
+    y : number or np.ndarray or Tensor or Pwc or Stf
         The denominator, :math:`y`.
     name : str, optional
         The name of the node. You can only provide a name if neither `x` nor `y` are Stfs.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise division :math:`x/y`.
 
     See Also
@@ -423,7 +375,7 @@ class TrueDivision(Node):
         forge.arg("x", type=NumericOrFunction),
         forge.arg("y", type=NumericOrFunction),
     ]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -433,28 +385,28 @@ class TrueDivision(Node):
 class FloorDivision(Node):
     r"""
     Calculates the element-wise rounded-down division between numbers, np.ndarrays, Tensors,
-    TensorPwcs, or Stfs. You can also use the arithmetic operator ``//`` to calculate their floor
+    Pwcs, or Stfs. You can also use the arithmetic operator ``//`` to calculate their floor
     division.
 
     Considering numbers and np.ndarrays as Tensors, if the two objects are of the same type,
-    so is the returned object. If the objects have different types, TensorPwcs and Stfs can operate
-    with a tensor (returning a TensorPwc or Stf, respectively).
+    so is the returned object. If the objects have different types, Pwcs and Stfs can operate
+    with a tensor (returning a Pwc or Stf, respectively).
 
     This operation supports broadcasting between the different objects. When operating a tensor-like
-    object with an Stf or TensorPwc, the time dimension of the latter is ignored.
+    object with an Stf or a Pwc, the time dimension of the latter is ignored.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The numerator, :math:`x`.
-    y : number or np.ndarray or Tensor or TensorPwc or Stf
+    y : number or np.ndarray or Tensor or Pwc or Stf
         The denominator, :math:`y`.
     name : str, optional
         The name of the node. You can only provide a name if neither `x` nor `y` are Stfs.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise rounded-down division :math:`\lfloor x/y \rfloor`.
 
     See Also
@@ -467,7 +419,7 @@ class FloorDivision(Node):
         forge.arg("x", type=NumericOrFunction),
         forge.arg("y", type=NumericOrFunction),
     ]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -476,28 +428,28 @@ class FloorDivision(Node):
 
 class Exponentiation(Node):
     """
-    Calculates the element-wise power between numbers, np.ndarrays, Tensors, TensorPwcs,
+    Calculates the element-wise power between numbers, np.ndarrays, Tensors, Pwcs,
     or Stfs. You can also use the arithmetic operator ``**`` to calculate their power.
 
     Considering numbers and np.ndarrays as Tensors, if the two objects are of the same type,
-    so is the returned object. If the objects have different types, TensorPwcs and Stfs can operate
-    with a tensor (returning a TensorPwc or Stf, respectively).
+    so is the returned object. If the objects have different types, Pwcs and Stfs can operate
+    with a tensor (returning a Pwc or Stf, respectively).
 
     This operation supports broadcasting between the different objects. When operating a tensor-like
-    object with an Stf or TensorPwc, the time dimension of the latter is ignored.
+    object with an Stf or a Pwc, the time dimension of the latter is ignored.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The base, :math:`x`.
-    y : number or np.ndarray or Tensor or TensorPwc or Stf
+    y : number or np.ndarray or Tensor or Pwc or Stf
         The exponent, :math:`y`.
     name : str, optional
         The name of the node. You can only provide a name if neither `x` nor `y` are Stfs.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise power :math:`x^y`.
 
     Warnings
@@ -512,7 +464,7 @@ class Exponentiation(Node):
         forge.arg("x", type=NumericOrFunction),
         forge.arg("y", type=NumericOrFunction),
     ]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -522,27 +474,27 @@ class Exponentiation(Node):
 class ComplexValue(Node):
     """
     Creates element-wise complex values from real numbers, np.ndarrays, Tensors,
-    TensorPwcs, or Stfs, that is, the real and imaginary parts.
+    Pwcs, or Stfs, that is, the real and imaginary parts.
 
     Considering numbers and np.ndarrays as Tensors, if the two objects are of the same type,
-    so is the returned object. If the objects have different types, TensorPwcs and Stfs can operate
-    with a tensor (returning a TensorPwc or Stf, respectively).
+    so is the returned object. If the objects have different types, Pwcs and Stfs can operate
+    with a tensor (returning a Pwc or Stf, respectively).
 
     This operation supports broadcasting between the different objects. When operating a tensor-like
-    object with an Stf or TensorPwc, the time dimension of the latter is ignored.
+    object with an Stf or a Pwc, the time dimension of the latter is ignored.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The real part, :math:`x`.
-    y : number or np.ndarray or Tensor or TensorPwc or Stf
+    y : number or np.ndarray or Tensor or Pwc or Stf
         The imaginary part, :math:`y`.
     name : str, optional
         The name of the node. You can only provide a name if neither `x` nor `y` are Stfs.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise complex number :math:`x+iy`.
 
     See Also
@@ -559,7 +511,7 @@ class ComplexValue(Node):
         forge.arg("x", type=NumericOrFunction),
         forge.arg("y", type=NumericOrFunction),
     ]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -569,11 +521,11 @@ class ComplexValue(Node):
 class Matmul(Node):
     """
     Calculates the matrix multiplication between np.ndarrays, Tensors,
-    TensorPwcs, or Stfs. You can also use the arithmetic operator ``@``
+    Pwcs, or Stfs. You can also use the arithmetic operator ``@``
     to calculate their matrix multiplication.
 
-    If any of the inputs is a TensorPwc or Stf, the output is also a
-    TensorPwc or Stf (mixing TensorPwcs and Stfs is not supported).
+    If any of the inputs is a Pwc or Stf, the output is also a
+    Pwc or Stf (mixing Pwcs and Stfs is not supported).
     Otherwise, the output is a Tensor.
 
     This operation supports broadcasting between the batch dimensions of
@@ -581,16 +533,16 @@ class Matmul(Node):
     innermost ones (where the matrix multiplication is performed) are
     considered batch dimensions.
 
-    When operating a tensor-like object with an Stf or TensorPwc, the time
+    When operating a tensor-like object with an Stf or a Pwc, the time
     dimension of the latter is ignored.
 
     Parameters
     ----------
-    x : np.ndarray or Tensor or TensorPwc or Stf
+    x : np.ndarray or Tensor or Pwc or Stf
         The left multiplicand. It must be a matrix (or batch of matrices)
         and its last dimension must be the same as the second-to-last
         dimension of `y`.
-    y : np.ndarray or Tensor or TensorPwc or Stf
+    y : np.ndarray or Tensor or Pwc or Stf
         The right multiplicand. It must be a matrix (or batch of matrices)
         and its second-to-last dimension must be the same as the last
         dimension of `x`.
@@ -600,26 +552,22 @@ class Matmul(Node):
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The matrix product of the input objects. If any of the input
-        objects is a TensorPwc or Stf, the returned objects has the same
+        objects is a Pwc or Stf, the returned objects has the same
         type. Otherwise, it is a Tensor.
     """
 
     name = "matmul"
     args = [
-        forge.arg(
-            "x", type=Union[np.ndarray, types.Tensor, types.TensorPwc, types.Stf]
-        ),
-        forge.arg(
-            "y", type=Union[np.ndarray, types.Tensor, types.TensorPwc, types.Stf]
-        ),
+        forge.arg("x", type=Union[np.ndarray, Tensor, Pwc, Stf]),
+        forge.arg("y", type=Union[np.ndarray, Tensor, Pwc, Stf]),
     ]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
-        def _validate_values_shape(x_shape, y_shape, x_name, y_name):
+        def _validate_value_shape(x_shape, y_shape, x_name, y_name):
             if len(x_shape) < 2 or len(y_shape) < 2:
                 raise QctrlException(
                     f"The shapes {x_shape} of {x_name} and {y_shape} of"
@@ -649,7 +597,7 @@ class Matmul(Node):
         return _create_flexible_binary_node_data(
             _operation,
             "matmul",
-            validate_values_shape=_validate_values_shape,
+            validate_value_shape=_validate_value_shape,
             **kwargs,
         )
 
@@ -657,10 +605,10 @@ class Matmul(Node):
 class Kron(Node):
     """
     Calculates the Kronecker product between np.ndarrays, Tensors,
-    TensorPwcs, or Stfs.
+    Pwcs, or Stfs.
 
-    If any of the inputs is a TensorPwc or Stf, the output is also a
-    TensorPwc or Stf (mixing TensorPwcs and Stfs is not supported).
+    If any of the inputs is a Pwc or Stf, the output is also a
+    Pwc or Stf (mixing Pwcs and Stfs is not supported).
     Otherwise, the output is a Tensor.
 
     This operation supports broadcasting between the batch dimensions of
@@ -668,14 +616,14 @@ class Kron(Node):
     innermost ones (where the Kronecker product is performed) are
     considered batch dimensions.
 
-    When operating a tensor-like object with an Stf or TensorPwc, the time
+    When operating a tensor-like object with an Stf or a Pwc, the time
     dimension of the latter is ignored.
 
     Parameters
     ----------
-    x : np.ndarray or Tensor or TensorPwc or Stf
+    x : np.ndarray or Tensor or Pwc or Stf
         The left multiplicand. It must be a have at least two dimensions.
-    y : np.ndarray or Tensor or TensorPwc or Stf
+    y : np.ndarray or Tensor or Pwc or Stf
         The right multiplicand. It must be a have at least two dimensions.
     name : str, optional
         The name of the node. You can only provide a name if neither `x`
@@ -683,26 +631,22 @@ class Kron(Node):
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The Kronecker product of the input objects. If any of the input
-        objects is a TensorPwc or Stf, the returned objects has the same
+        objects is a Pwc or Stf, the returned objects has the same
         type. Otherwise, it is a Tensor.
     """
 
     name = "kron"
     args = [
-        forge.arg(
-            "x", type=Union[np.ndarray, types.Tensor, types.TensorPwc, types.Stf]
-        ),
-        forge.arg(
-            "y", type=Union[np.ndarray, types.Tensor, types.TensorPwc, types.Stf]
-        ),
+        forge.arg("x", type=Union[np.ndarray, Tensor, Pwc, Stf]),
+        forge.arg("y", type=Union[np.ndarray, Tensor, Pwc, Stf]),
     ]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
-        def _validate_values_shape(x_shape, y_shape, x_name, y_name):
+        def _validate_value_shape(x_shape, y_shape, x_name, y_name):
             if len(x_shape) < 2 or len(y_shape) < 2:
                 raise QctrlException(
                     f"The shapes {x_shape} of {x_name} and {y_shape} of"
@@ -728,6 +672,6 @@ class Kron(Node):
         return _create_flexible_binary_node_data(
             _operation,
             "kron",
-            validate_values_shape=_validate_values_shape,
+            validate_value_shape=_validate_value_shape,
             **kwargs,
         )
