@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -19,11 +19,10 @@ from typing import (
 import forge
 import numpy as np
 
-from qctrlcommons.node import types
 from qctrlcommons.node.base import Node
 from qctrlcommons.node.node_data import (
-    TensorNodeData,
-    TensorPwcNodeData,
+    Pwc,
+    Tensor,
 )
 from qctrlcommons.node.utils import (
     validate_ms_shapes,
@@ -47,7 +46,7 @@ class MsPhases(Node):
 
     Parameters
     ----------
-    drives : list[TensorPwc(1D, complex)]
+    drives : list[Pwc(1D, complex)]
         A list of piecewise-constant drives :math:`\{\gamma_j\}`.
         The number of drives must be the same as the number of ions :math:`N`.
         Drive values must be in rad/s and drive durations must be in seconds.
@@ -141,12 +140,12 @@ class MsPhases(Node):
 
     name = "ms_phases"
     args = [
-        forge.arg("drives", type=List[types.TensorPwc]),
+        forge.arg("drives", type=List[Pwc]),
         forge.arg("lamb_dicke_parameters", type=np.ndarray),
         forge.arg("relative_detunings", type=np.ndarray),
         forge.arg("sample_times", type=Optional[np.ndarray], default=None),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -156,8 +155,8 @@ class MsPhases(Node):
         relative_detunings = kwargs.get("relative_detunings")
         check_argument_iteratable(drives, "drives")
         check_argument(
-            all(isinstance(drive, TensorPwcNodeData) for drive in drives),
-            "Each of the drives must be a TensorPwc.",
+            all(isinstance(drive, Pwc) for drive in drives),
+            "Each of the drives must be a Pwc.",
             {"drives": drives},
         )
         ion_count = len(drives)
@@ -173,7 +172,7 @@ class MsPhases(Node):
             check_sample_times(sample_times, "sample_times")
             time_count = len(sample_times)
             shape = (time_count,) + tuple(shape)
-        return TensorNodeData(_operation, shape=shape)
+        return Tensor(_operation, shape=shape)
 
 
 class MsDisplacements(Node):
@@ -186,7 +185,7 @@ class MsDisplacements(Node):
 
     Parameters
     ----------
-    drives : list[TensorPwc(1D, complex)]
+    drives : list[Pwc(1D, complex)]
         A list of piecewise-constant drives :math:`\{\gamma_j\}`.
         The number of drives must be the same as the number of ions :math:`N`.
         Drive values must be in rad/s and drive durations must be in seconds.
@@ -286,12 +285,12 @@ class MsDisplacements(Node):
 
     name = "ms_displacements"
     args = [
-        forge.arg("drives", type=List[types.TensorPwc]),
+        forge.arg("drives", type=List[Pwc]),
         forge.arg("lamb_dicke_parameters", type=np.ndarray),
         forge.arg("relative_detunings", type=np.ndarray),
         forge.arg("sample_times", type=Optional[np.ndarray], default=None),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -301,8 +300,8 @@ class MsDisplacements(Node):
         relative_detunings = kwargs.get("relative_detunings")
         check_argument_iteratable(drives, "drives")
         check_argument(
-            all(isinstance(drive, TensorPwcNodeData) for drive in drives),
-            "Each of the drives must be a TensorPwc.",
+            all(isinstance(drive, Pwc) for drive in drives),
+            "Each of the drives must be a Pwc.",
             {"drives": drives},
         )
         ion_count = len(drives)
@@ -318,7 +317,7 @@ class MsDisplacements(Node):
             check_sample_times(sample_times, "sample_times")
             time_count = len(sample_times)
             shape = (time_count,) + tuple(shape)
-        return TensorNodeData(_operation, shape=shape)
+        return Tensor(_operation, shape=shape)
 
 
 class MsInfidelity(Node):
@@ -388,12 +387,12 @@ class MsInfidelity(Node):
 
     name = "ms_infidelity"
     args = [
-        forge.arg("phases", type=types.Tensor),
-        forge.arg("displacements", type=types.Tensor),
+        forge.arg("phases", type=Tensor),
+        forge.arg("displacements", type=Tensor),
         forge.arg("target_phases", type=np.ndarray),
         forge.arg("mean_phonon_numbers", type=Optional[np.ndarray], default=None),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -462,7 +461,7 @@ class MsInfidelity(Node):
                 },
             )
         shape = phases_shape[:-2]
-        return TensorNodeData(_operation, shape=shape)
+        return Tensor(_operation, shape=shape)
 
 
 class MsDephasingRobustCost(Node):
@@ -477,7 +476,7 @@ class MsDephasingRobustCost(Node):
 
     Parameters
     ----------
-    drives : list[TensorPwc(1D, complex)]
+    drives : list[Pwc(1D, complex)]
         The list of piecewise-constant drives :math:`\{\gamma_j\}`. The
         number of drives must be the same as the number of ions :math:`N`.
         Drive values must be in rad/s and drive durations must be in
@@ -564,11 +563,11 @@ class MsDephasingRobustCost(Node):
     """
     name = "ms_dephasing_robust_cost"
     args = [
-        forge.arg("drives", type=List[types.TensorPwc]),
+        forge.arg("drives", type=List[Pwc]),
         forge.arg("lamb_dicke_parameters", type=np.ndarray),
         forge.arg("relative_detunings", type=np.ndarray),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -577,8 +576,8 @@ class MsDephasingRobustCost(Node):
         relative_detunings = kwargs.get("relative_detunings")
         check_argument_iteratable(drives, "drives")
         check_argument(
-            all(isinstance(drive, TensorPwcNodeData) for drive in drives),
-            "Each of the drives must be a TensorPwc.",
+            all(isinstance(drive, Pwc) for drive in drives),
+            "Each of the drives must be a Pwc.",
             {"drives": drives},
         )
         ion_count = len(drives)
@@ -589,4 +588,4 @@ class MsDephasingRobustCost(Node):
             rd_values=relative_detunings,
             rd_name="relative_detunings",
         )
-        return TensorNodeData(_operation, shape=())
+        return Tensor(_operation, shape=())
