@@ -13,8 +13,7 @@
 """
 Module for nodes related to open quantum systems.
 """
-
-
+import warnings
 from typing import (
     List,
     Optional,
@@ -27,6 +26,7 @@ import numpy as np
 from scipy.sparse import coo_matrix
 
 from qctrlcommons.node.base import Node
+from qctrlcommons.node.deprecation import deprecated_node
 from qctrlcommons.node.node_data import (
     Pwc,
     SparsePwc,
@@ -45,68 +45,13 @@ from qctrlcommons.preconditions import (
 )
 
 
+@deprecated_node(
+    extra_info="Note that `krylov_subspace_dimension` parameter is deprecated from "
+    "density_matrix_evolution_pwc, meaning you don't need to call this node to "
+    "estimate a Krylov subspace dimension."
+)  # deprecated 2021/09/09
 class EstimatedKrylovSubspaceDimensionArnoldi(Node):
-    r"""
-    Calculates an appropriate Krylov subspace dimension (:math:`k`) to use in the Arnoldi
-    integrator while keeping the total error in the evolution below a given error tolerance.
-
-    Parameters
-    ----------
-    hamiltonian_sample : Union[Tensor, np.ndarray, scipy.sparse.spmatrix]
-        A Hamiltonian, a sample with couplings similar to those passed to the Arnoldi integrator.
-    lindblad_terms : List[Tuple[float, Union[np.ndarray, sp.sparse.spmatrix]]]
-        A list of pairs of constant positive decay rates and Lindblad operators with Hilbert space
-        dimension D. If mixed types of operators are used, all operators get transformed into
-        dense operators.
-    duration : float
-        The total evolution time.
-    maximum_segment_duration : float
-        The maximum duration of the piecewise-constant Hamiltonian segments.
-    error_tolerance : float, optional
-        Tolerance for the error in the whole integration (defined as the Frobenius norm of the
-        difference with the exact solution).
-        Defaults to 1e-6.
-    seed : int, optional
-        This function uses a random number generator which may be seeded with this int.
-        If None, this function's result may vary between calls.
-    name : str, optional
-        The name of the node.
-
-    Returns
-    -------
-    Tensor
-        Recommended value of :math:`k` to use in an Arnoldi integration with a Liouvillian from a
-        Hamiltonian and Lindblad terms similar to the ones passed.
-
-    See Also
-    --------
-    density_matrix_evolution_pwc : Evolve a quantum state in an open system.
-
-    Notes
-    -----
-    To provide the recommended :math:`k` parameter, this function uses the bound in the error for
-    the Arnoldi algorithm [1]_ [2]_ as an estimate for the error. For a single time step this gives
-
-    .. math::
-        \mathrm{error} \leq 12 e^{-\rho\tau} \left (\frac{e \rho \tau}{k}  \right )^{k}
-
-    where :math:`\tau` is the time step and :math:`\rho` is a positive number such that the
-    numerical range of the generator is contained in a disc of radius :math:`\rho` centered at
-    :math:`(-\rho,0)` of the complex plane.
-
-    As this bound overestimates the error, the actual resulting errors with the recommended
-    parameter are expected to be (a few orders of magnitude) smaller than the requested tolerance.
-
-    References
-    ----------
-    .. [1] `N. Del Buono and L. Lopez,
-           Lecture Notes in Computer Science 2658, 111 (2003).
-           <https://doi.org/10.1007/3-540-44862-4_13>`_
-
-    .. [2] `M. Hochbruck and C. Lubich,
-           SIAM Journal on Numerical Analysis 34, 1911 (1997).
-           <https://doi.org/10.1137/S0036142995280572>`_
-    """
+    """Deprecated node."""
 
     name = "estimated_krylov_subspace_dimension_arnoldi"
     args = [
@@ -143,6 +88,7 @@ class EstimatedKrylovSubspaceDimensionArnoldi(Node):
         return Tensor(_operation, shape=())
 
 
+# krylov_subspace_dimension parameter deprecated 2021/09/09
 class DensityMatrixEvolutionPwc(Node):
     r"""
     Calculates the state evolution of an open system described by the GKS–Lindblad master
@@ -154,17 +100,22 @@ class DensityMatrixEvolutionPwc(Node):
     You may need to increase the number of segments that you choose for the
     discretization depending on the sizes of oscillations in the smooth controls.
 
+    By default, this function computes the exact piecewise constant solution and might be
+    inefficient for simulating large systems. In that case, you may use an approximated
+    method by passing the `error_tolerance` parameter, which controls the computation
+    precision.
+
     Parameters
     ----------
-    initial_density_matrix : Union[np.ndarray, Tensor]
+    initial_density_matrix : np.ndarray or Tensor
         A 2D array of the shape ``[D, D]`` representing the initial density matrix of
         the system, :math:`\rho_{\rm s}`. You can also pass a batch of density matrices
         and the input data shape must be ``[B, D, D]`` where ``B`` is the batch dimension.
-    hamiltonian : Union[Pwc, SparsePwc]
+    hamiltonian : Pwc or SparsePwc
         A piecewise-constant function representing the effective system Hamiltonian,
         :math:`H_{\rm s}(t)`, for the entire evolution duration. If you pass any Lindblad operator
         as a dense array, the Hamiltonian will get converted to a (dense) Pwc.
-    lindblad_terms : List[Tuple[float, Union[np.ndarray, scipy.sparse.coo_matrix]]]
+    lindblad_terms : list[tuple[float, np.ndarray or scipy.sparse.coo_matrix]]
         A list of pairs, :math:`(\gamma_j, L_j)`, representing the positive decay rate
         :math:`\gamma_j` and the Lindblad operator :math:`L_j` for each coupling
         channel :math:`j`. If you pass the Hamiltonian as a Pwc, the operators will get
@@ -172,11 +123,21 @@ class DensityMatrixEvolutionPwc(Node):
     sample_times : np.ndarray, optional
         A 1D array of length :math:`T` specifying the times :math:`\{t_i\}` at which this
         function calculates system states. Must be ordered and contain at least one element.
-    krylov_subspace_dimension : Union[Tensor, int], optional
-        The dimension of the Krylov subspace `k` for the Arnoldi method, which allows a more
-        efficient calculation for large Hilbert spaces. A larger value leads to
-        a better precision at the cost of longer computational time. If omitted or None
-        it calculates the exact piecewise constant solution.
+        Note that increasing the density of sample times does not affect the computation precision
+        of this function.
+    krylov_subspace_dimension : deprecated
+        This parameter is to be deprecated and has no effect on the calculation.
+        Assigning a value to this parameter is equivalent to setting `error_tolerance` to 1e-7.
+    error_tolerance : float, optional
+        Defaults to `None`, meaning this function calculates the exact piecewise constant solution.
+        Use this option to invoke an approximation method to solve the master equation.
+        This tolerance controls the computation precision. That is, the 2-norm of the
+        difference between the propagated state and the exact solution
+        at the final time (and at each sample time if passed) is within the error tolerance.
+        Note that, if set, this value must be smaller than 1e-2 (inclusive). However, setting
+        it to a too small value (for example below 1e-12) might result in slower
+        computation, but would not further improve the precision, since the dominating error in
+        that case is due to floating point error. A recommended value is around 1e-7.
     name : str, optional
         The name of the node.
 
@@ -190,8 +151,6 @@ class DensityMatrixEvolutionPwc(Node):
     See Also
     --------
     discretize_stf : Discretize an `Stf` into a `Pwc`.
-    estimated_krylov_subspace_dimension_arnoldi : Obtain a Krylov subspace dimension to use
-        with this integrator.
     sparse_pwc_operator : Create `SparsePwc` operators.
     state_evolution_pwc : Corresponding operation for coherent evolution.
 
@@ -213,14 +172,8 @@ class DensityMatrixEvolutionPwc(Node):
 
     for any system operator :math:`X`.
 
-    This function calculates the system state at the sample times :math:`\{t_j\}` that you provide
-    by integrating the GKS–Lindblad equation via the Arnoldi algorithm. Note that increasing the
-    density of sample times does not affect the accuracy of the integration. However, increasing the
-    Krylov subspace dimension or subdividing the Hamiltonian into shorter piecewise-constant
-    segments can reduce the integration error, at the expense of longer computation times.
-
-    Moreover, this function uses sparse matrix multiplication when the Hamiltonian is passed as a
-    `SparsePwc` and the Lindblad operators as sparse matrices. This can lead to more efficient
+    This function uses sparse matrix multiplication when the Hamiltonian is passed as a
+    `SparsePwc` and the Lindblad operators as sparse matrices. This leads to more efficient
     calculations when they involve large operators that are relatively sparse (contain mostly
     zeros). In this case, the initial density matrix is still a densely represented array or tensor.
 
@@ -245,6 +198,7 @@ class DensityMatrixEvolutionPwc(Node):
             type=Union[int, Tensor, None],
             default=None,
         ),
+        forge.arg("error_tolerance", type=Optional[float], default=None),
     ]
     rtype = Tensor
 
@@ -254,6 +208,8 @@ class DensityMatrixEvolutionPwc(Node):
         initial_density_matrix = kwargs.get("initial_density_matrix")
         hamiltonian = kwargs.get("hamiltonian")
         lindblad_terms = kwargs.get("lindblad_terms")
+        error_tolerance = kwargs.get("error_tolerance")
+        krylov_subspace_dimension = kwargs.get("krylov_subspace_dimension")
 
         check_argument(
             isinstance(hamiltonian, (Pwc, SparsePwc)),
@@ -265,6 +221,20 @@ class DensityMatrixEvolutionPwc(Node):
             "Initial density matrix must be a NumPy array or a Tensor.",
             {"initial_density_matrix": initial_density_matrix},
         )
+
+        if krylov_subspace_dimension is not None:
+            warnings.warn(
+                "The parameter `krylov_subspace_dimension` of `density_matrix_evolution_pwc`"
+                "will be removed in the future. Please use `error_tolerance` instead."
+            )
+            krylov_subspace_dimension = None
+            error_tolerance = 1e-7  # in case users pass dim and tol together
+        if error_tolerance is not None:
+            check_argument(
+                error_tolerance <= 1e-2,
+                "`error_tolerance` must not be greater than 1e-2.",
+                {"error_tolerance": error_tolerance},
+            )
         if sample_times is not None:
             check_sample_times(sample_times, "sample_times")
         check_density_matrix_shape(initial_density_matrix, "initial_density_matrix")
