@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -20,15 +20,16 @@ from typing import (
 import forge
 import numpy as np
 
-from qctrlcommons.node import types
 from qctrlcommons.node.base import Node
+from qctrlcommons.node.deprecation import deprecated_node
 from qctrlcommons.node.node_data import (
-    TensorNodeData,
-    TensorPwcNodeData,
+    Pwc,
+    Tensor,
 )
 from qctrlcommons.node.utils import (
-    validate_batch_and_values_shapes,
+    validate_batch_and_value_shapes,
     validate_hamiltonian,
+    validate_inputs_real_fourier_signal,
     validate_shape,
 )
 from qctrlcommons.preconditions import (
@@ -42,7 +43,7 @@ from qctrlcommons.preconditions import (
 )
 
 
-class TensorPwc(Node):
+class PwcOperation(Node):
     r"""
     Creates a piecewise-constant function of time.
 
@@ -70,30 +71,30 @@ class TensorPwc(Node):
 
     Returns
     -------
-    TensorPwc
+    Pwc
         The piecewise-constant function of time :math:`v(t)`, satisfying
         :math:`v(t)=v_n` for :math:`t_{n-1}\leq t\leq t_n`, where
         :math:`t_0=0` and :math:`t_n=t_{n-1}+\delta t_n`. If you provide a
-        batch of values, the returned `TensorPwc` represents a
+        batch of values, the returned `Pwc` represents a
         corresponding batch of :math:`B_1 \times \ldots \times B_n`
         functions :math:`v(t)`, each of shape
         :math:`D_1 \times \ldots \times D_m`.
 
     See Also
     --------
-    pwc_operator : Create `TensorPwc` operators.
-    pwc_signal : Create `TensorPwc` signals from (possibly complex) values.
-    pwc_sum : Sum multiple `TensorPwc`s.
+    pwc_operator : Create `Pwc` operators.
+    pwc_signal : Create `Pwc` signals from (possibly complex) values.
+    pwc_sum : Sum multiple `Pwc`\s.
     """
 
-    name = "tensor_pwc"
+    name = "pwc"
     args = [
         forge.arg("durations", type=np.ndarray),
-        forge.arg("values", type=Union[np.ndarray, types.Tensor]),
+        forge.arg("values", type=Union[np.ndarray, Tensor]),
         forge.arg("time_dimension", type=int, default=0),
     ]
 
-    rtype = types.TensorPwc
+    rtype = Pwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -103,7 +104,7 @@ class TensorPwc(Node):
         check_argument_numeric(durations, "durations")
         check_argument_numeric(values, "values")
         check_argument_integer(time_dimension, "time_dimension")
-        values_shape = validate_shape(values, "values")
+        value_shape = validate_shape(values, "values")
         durations_shape = validate_shape(durations, "durations")
         check_argument(
             len(durations_shape) == 1,
@@ -111,23 +112,39 @@ class TensorPwc(Node):
             {"durations": durations},
         )
         check_argument(
-            values_shape[time_dimension] == durations_shape[0],
+            -len(value_shape) <= time_dimension < len(value_shape),
+            "The `time_dimension` must be a valid `values` dimension.",
+            {"time_dimension": time_dimension, "values": values},
+        )
+        check_argument(
+            value_shape[time_dimension] == durations_shape[0],
             "The size of the `time_dimension` of `values` must be equal to the length"
             " of `durations`.",
-            {"values": values, "durations": durations},
+            {
+                "time_dimension": time_dimension,
+                "values": values,
+                "durations": durations,
+            },
         )
         for index, duration in enumerate(durations):
             check_duration(duration, f"durations[{index}]")
         if time_dimension < 0:
-            time_dimension = len(values_shape) + time_dimension
-        return TensorPwcNodeData(
+            time_dimension = len(value_shape) + time_dimension
+        return Pwc(
             _operation,
-            values_shape=values_shape[time_dimension + 1 :],
+            value_shape=value_shape[time_dimension + 1 :],
             durations=durations,
-            batch_shape=values_shape[:time_dimension],
+            batch_shape=value_shape[:time_dimension],
         )
 
 
+@deprecated_node("pwc")  # deprecated 2021/05/20
+class TensorPwc(PwcOperation):
+    """Deprecated node."""
+
+    name = "tensor_pwc"
+
+
 class PwcSignal(Node):
     r"""
     Creates a piecewise-constant signal (scalar-valued function of time).
@@ -151,29 +168,29 @@ class PwcSignal(Node):
 
     Returns
     -------
-    TensorPwc
+    Pwc
         The piecewise-constant function of time :math:`\alpha(t)`, satisfying
         :math:`\alpha(t)=\alpha_n` for :math:`t_{n-1}\leq t\leq t_n`, where
         :math:`t_n=n\tau/N` (where :math:`N` is the number of values
         in :math:`\{\alpha_n\}`). If you provide a batch of values, the
-        returned `TensorPwc` represents a corresponding batch of
+        returned `Pwc` represents a corresponding batch of
         :math:`B_1 \times \ldots \times B_n` functions :math:`\alpha(t)`.
 
     See Also
     --------
-    complex_pwc_signal : Create complex `TensorPwc` signals from their moduli and phases.
-    pwc_operator : Create `TensorPwc` operators.
-    pwc_sum : Sum multiple `TensorPwc`s.
-    symmetrize_pwc : Symmetrize `TensorPwc`s.
-    tensor_pwc : Corresponding operation with support for segments of different durations.
+    complex_pwc_signal : Create complex `Pwc` signals from their moduli and phases.
+    pwc : Corresponding operation with support for segments of different durations.
+    pwc_operator : Create `Pwc` operators.
+    pwc_sum : Sum multiple `Pwc`\s.
+    symmetrize_pwc : Symmetrize `Pwc`\s.
     """
 
     name = "pwc_signal"
     args = [
-        forge.arg("values", type=Union[np.ndarray, types.Tensor]),
+        forge.arg("values", type=Union[np.ndarray, Tensor]),
         forge.arg("duration", type=float),
     ]
-    rtype = types.TensorPwc
+    rtype = Pwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -188,9 +205,9 @@ class PwcSignal(Node):
             {"values": values},
         )
         durations = duration / shape[-1] * np.ones(shape[-1])
-        return TensorPwcNodeData(
+        return Pwc(
             _operation,
-            values_shape=(),
+            value_shape=(),
             durations=durations,
             batch_shape=shape[:-1],
         )
@@ -224,27 +241,27 @@ class ComplexPwcSignal(Node):
 
     Returns
     -------
-    TensorPwc(complex)
+    Pwc(complex)
         The piecewise-constant function of time :math:`v(t)`, satisfying
         :math:`v(t)=\Omega_n e^{i\phi_n}` for :math:`t_{n-1}\leq t\leq t_n`,
         where :math:`t_n=n\tau/N` (where :math:`N` is the number of
         values in :math:`\{\Omega_n\}` and :math:`\{\phi_n\}`). If you
-        provide a batch of `moduli` and `phases`, the returned `TensorPwc`
+        provide a batch of `moduli` and `phases`, the returned `Pwc`
         represents a corresponding batch of
         :math:`B_1 \times \ldots \times B_n` functions :math:`v(t)`.
 
     See Also
     --------
-    pwc_signal : Create `TensorPwc` signals from (possibly complex) values.
+    pwc_signal : Create `Pwc` signals from (possibly complex) values.
     """
 
     name = "complex_pwc_signal"
     args = [
-        forge.arg("moduli", type=Union[np.ndarray, types.Tensor]),
-        forge.arg("phases", type=Union[np.ndarray, types.Tensor]),
+        forge.arg("moduli", type=Union[np.ndarray, Tensor]),
+        forge.arg("phases", type=Union[np.ndarray, Tensor]),
         forge.arg("duration", type=float),
     ]
-    rtype = types.TensorPwc
+    rtype = Pwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -267,21 +284,21 @@ class ComplexPwcSignal(Node):
             {"moduli": moduli, "phases": phases},
         )
         durations = duration / moduli_shape[-1] * np.ones(moduli_shape[-1])
-        return TensorPwcNodeData(
+        return Pwc(
             _operation,
-            values_shape=(),
+            value_shape=(),
             durations=durations,
             batch_shape=moduli_shape[:-1],
         )
 
 
 class PwcOperator(Node):
-    """
+    r"""
     Creates a constant operator multiplied by a piecewise-constant signal.
 
     Parameters
     ----------
-    signal : TensorPwc
+    signal : Pwc
         The piecewise-constant signal :math:`a(t)`, or a batch of
         piecewise-constant signals.
     operator : np.ndarray or Tensor
@@ -291,50 +308,59 @@ class PwcOperator(Node):
 
     Returns
     -------
-    TensorPwc
+    Pwc
         The piecewise-constant operator :math:`a(t)A` (or a batch of
         piecewise-constant operators, if you provide a batch of
         piecewise-constant signals).
 
     See Also
     --------
-    complex_pwc_signal : Create complex `TensorPwc` signals from their moduli and phases.
-    constant_pwc_operator : Create constant `TensorPwc`s.
+    complex_pwc_signal : Create complex `Pwc` signals from their moduli and phases.
+    constant_pwc_operator : Create constant `Pwc`\s.
+    pwc : Create piecewise-constant functions.
     pwc_operator_hermitian_part : Hermitian part of a piecewise-constant operator.
-    pwc_signal : Create `TensorPwc` signals from (possibly complex) values.
-    pwc_sum : Sum multiple `TensorPwc`s.
-    stf_operator : Corresponding operation for `Stf`s.
-    tensor_pwc : Create piecewise-constant functions.
+    pwc_signal : Create `Pwc` signals from (possibly complex) values.
+    pwc_sum : Sum multiple `Pwc`\s.
+    stf_operator : Corresponding operation for `Stf`\s.
+    stf_operator_hermitian_part : Corresponding operation for `Stf`\s.
     """
 
     name = "pwc_operator"
     args = [
-        forge.arg("signal", type=types.TensorPwc),
-        forge.arg("operator", type=Union[np.ndarray, types.Tensor]),
+        forge.arg("signal", type=Pwc),
+        forge.arg("operator", type=Union[np.ndarray, Tensor]),
     ]
-    rtype = types.TensorPwc
+    rtype = Pwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         signal = kwargs.get("signal")
         operator = kwargs.get("operator")
         check_argument(
-            isinstance(signal, TensorPwcNodeData),
-            "The signal must be a TensorPwc.",
+            isinstance(signal, Pwc),
+            "The signal must be a Pwc.",
             {"signal": signal},
         )
-        batch_shape, _ = validate_batch_and_values_shapes(signal, "signal")
-        values_shape = validate_shape(operator, "operator")
+        batch_shape, signal_value_shape = validate_batch_and_value_shapes(
+            signal, "signal"
+        )
+        check_argument(
+            signal_value_shape == (),
+            "The signal must be scalar-valued.",
+            {"signal": signal},
+        )
+
+        value_shape = validate_shape(operator, "operator")
         check_operator(operator, "operator")
         check_argument(
-            len(values_shape) == 2,
+            len(value_shape) == 2,
             "The operator must be a matrix, not a batch.",
             {"operator": operator},
-            extras={"operator.shape": values_shape},
+            extras={"operator.shape": value_shape},
         )
-        return TensorPwcNodeData(
+        return Pwc(
             _operation,
-            values_shape=values_shape,
+            value_shape=value_shape,
             durations=signal.durations,
             batch_shape=batch_shape,
         )
@@ -357,23 +383,23 @@ class ConstantPwcOperator(Node):
 
     Returns
     -------
-    TensorPwc
+    Pwc
         The constant operator :math:`t\mapsto A` (for :math:`0\leq t\leq\tau`)
         (or a batch of constant operators, if you provide a batch of operators).
 
     See Also
     --------
-    constant_stf_operator : Corresponding operation for `Stf`s.
-    pwc_operator : Create `TensorPwc` operators.
+    constant_stf_operator : Corresponding operation for `Stf`\s.
+    pwc_operator : Create `Pwc` operators.
     pwc_operator_hermitian_part : Hermitian part of a piecewise-constant operator.
     """
 
     name = "constant_pwc_operator"
     args = [
         forge.arg("duration", type=float),
-        forge.arg("operator", type=Union[np.ndarray, types.Tensor]),
+        forge.arg("operator", type=Union[np.ndarray, Tensor]),
     ]
-    rtype = types.TensorPwc
+    rtype = Pwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -382,9 +408,9 @@ class ConstantPwcOperator(Node):
         check_duration(duration, "duration")
         shape = validate_shape(operator, "operator")
         check_operator(operator, "operator")
-        return TensorPwcNodeData(
+        return Pwc(
             _operation,
-            values_shape=shape[-2:],
+            value_shape=shape[-2:],
             durations=np.array([duration]),
             batch_shape=shape[:-2],
         )
@@ -396,7 +422,7 @@ class ConstantPwc(Node):
 
     Parameters
     ----------
-    constant : np.ndarray or Tensor
+    constant : number or np.ndarray or Tensor
         The value :math:`c` of the function on the constant segment.
         To create a batch of :math:`B_1 \times \ldots \times B_n` piecewise-constant
         functions of shape :math:`D_1 \times \ldots \times D_m`, provide this `constant`
@@ -413,24 +439,24 @@ class ConstantPwc(Node):
 
     Returns
     -------
-    TensorPwc
+    Pwc
         The constant function :math:`f(t) = c` (for :math:`0\leq t\leq\tau`)
         (or a batch of constant functions, if you provide `batch_dimension_count`).
 
     See Also
     --------
-    constant_pwc_operator : Create constant `TensorPwc` operators.
-    constant_stf : Corresponding operation for `Stf`s.
-    tensor_pwc : Create piecewise-constant functions.
+    constant_pwc_operator : Create constant `Pwc` operators.
+    constant_stf : Corresponding operation for `Stf`\s.
+    pwc : Create piecewise-constant functions.
     """
 
     name = "constant_pwc"
     args = [
-        forge.arg("constant", type=Union[np.ndarray, types.Tensor]),
+        forge.arg("constant", type=Union[float, complex, np.ndarray, Tensor]),
         forge.arg("duration", type=float),
         forge.arg("batch_dimension_count", type=int, default=0),
     ]
-    rtype = types.TensorPwc
+    rtype = Pwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -445,20 +471,20 @@ class ConstantPwc(Node):
         shape = validate_shape(constant, "constant")
         check_argument(
             len(shape) >= batch_dimension_count,
-            "Number of batch dimensions must not be larger than number of dimensions "
-            "of the input constant.",
+            "The number of batch dimensions must not be larger than the number "
+            "of dimensions of the input constant.",
             {"constant": constant, "batch_dimension_count": batch_dimension_count},
-            {"Number of value dimensions": len(constant.shape)},
+            {"Number of value dimensions": len(shape)},
         )
         check_argument(
             batch_dimension_count >= 0,
-            "Number of batch dimensions must not be negative.",
+            "The number of batch dimensions must not be negative.",
             {"batch_dimension_count": batch_dimension_count},
         )
 
-        return TensorPwcNodeData(
+        return Pwc(
             _operation,
-            values_shape=shape[batch_dimension_count:],
+            value_shape=shape[batch_dimension_count:],
             durations=np.array([duration]),
             batch_shape=shape[:batch_dimension_count],
         )
@@ -470,43 +496,44 @@ class PwcOperatorHermitianPart(Node):
 
     Parameters
     ----------
-    operator : TensorPwc(3D)
-        The operator :math:`A(t)`.
+    operator : Pwc
+        The operator :math:`A(t)`, or a batch of operators.
     name : str, optional
         The name of the node.
 
     Returns
     -------
-    TensorPwc(3D)
-        The Hermitian part :math:`\frac{1}{2}(A(t)+A^\dagger(t))`.
+    Pwc
+        The Hermitian part :math:`\frac{1}{2}(A(t)+A^\dagger(t))`
+        (or a batch of Hermitian parts, if you provide a batch of operators).
 
     See Also
     --------
-    constant_pwc_operator : Create constant `TensorPwc`s.
-    pwc_operator : Create `TensorPwc` operators.
+    constant_pwc_operator : Create constant `Pwc`\s.
+    pwc_operator : Create `Pwc` operators.
     """
 
     name = "pwc_operator_hermitian_part"
     args = [
-        forge.arg("operator", type=types.TensorPwc),
+        forge.arg("operator", type=Pwc),
     ]
-    rtype = types.TensorPwc
+    rtype = Pwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         operator = kwargs.get("operator")
         check_argument(
-            isinstance(operator, TensorPwcNodeData),
-            "The operator must be a TensorPwc.",
+            isinstance(operator, Pwc),
+            "The operator must be a Pwc.",
             {"operator": operator},
         )
-        batch_shape, values_shape = validate_batch_and_values_shapes(
+        batch_shape, value_shape = validate_batch_and_value_shapes(
             operator,
             "operator",
         )
-        return TensorPwcNodeData(
+        return Pwc(
             _operation,
-            values_shape=values_shape,
+            value_shape=value_shape,
             durations=operator.durations,
             batch_shape=batch_shape,
         )
@@ -518,56 +545,55 @@ class PwcSum(Node):
 
     Parameters
     ----------
-    terms : list[TensorPwc]
+    terms : list[Pwc]
         The individual piecewise-constant terms :math:`\{v_j(t)\}` to sum. All
         terms must have the same duration, values of the same shape, and the
         same batch shape, but may have different segmentations (different
         numbers of segments of different durations) and different dtypes of
         segment values.
-
     name : str, optional
         The name of the node.
 
     Returns
     -------
-    TensorPwc
+    Pwc
         The piecewise-constant function (or batch of functions) of time
         :math:`\sum_j v_j(t)`. Its values have the same shape as the values of
         each of the `terms` that you provided. If each of the `terms` represents
         a batch of functions, this result represents a batch of functions with
         the same batch shape. If any `term` has complex-valued segments, the
-        value of the returned TensorPwc is complex, otherwise is float.
+        value of the returned Pwc is complex, otherwise is float.
 
     See Also
     --------
-    discretize_stf : Discretize an `Stf` into a `TensorPwc`.
-    pwc_operator : Create `TensorPwc` operators.
-    pwc_signal : Create `TensorPwc` signals from (possibly complex) values.
-    stf_sum : Corresponding operation for `Stf`s.
-    tensor_pwc : Create piecewise-constant functions.
+    discretize_stf : Discretize an `Stf` into a `Pwc`.
+    pwc : Create piecewise-constant functions.
+    pwc_operator : Create `Pwc` operators.
+    pwc_signal : Create `Pwc` signals from (possibly complex) values.
+    stf_sum : Corresponding operation for `Stf`\s.
     """
 
     name = "pwc_sum"
-    args = [forge.arg("terms", type=List[types.TensorPwc])]
-    rtype = types.TensorPwc
+    args = [forge.arg("terms", type=List[Pwc])]
+    rtype = Pwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         terms = kwargs.get("terms")
         check_argument_iteratable(terms, "terms")
         check_argument(
-            all(isinstance(term, TensorPwcNodeData) for term in terms),
-            "Each of the terms must be a TensorPwc.",
+            all(isinstance(term, Pwc) for term in terms),
+            "Each of the terms must be a Pwc.",
             {"terms": terms},
         )
-        batch_shape, values_shape = validate_batch_and_values_shapes(
+        batch_shape, value_shape = validate_batch_and_value_shapes(
             terms[0],
             "terms[0]",
         )
         check_argument(
             all(
                 (
-                    (term.values_shape == values_shape)
+                    (term.value_shape == value_shape)
                     and (term.batch_shape == batch_shape)
                 )
                 for term in terms[1:]
@@ -597,9 +623,9 @@ class PwcSum(Node):
                 0,
             )
         )
-        return TensorPwcNodeData(
+        return Pwc(
             _operation,
-            values_shape=values_shape,
+            value_shape=value_shape,
             durations=upsampled_durations,
             batch_shape=batch_shape,
         )
@@ -613,18 +639,11 @@ class RealFourierPwcSignal(Node):
     (sine/cosine) basis signals that can be optimized by varying their
     coefficients and, optionally, their frequencies.
 
-    This function supports three different modes: fixed frequencies (you
-    directly provide the frequencies for the basis signals), optimizable
-    frequencies (you provide the number of frequencies, and the frequency
-    values can be tuned by the optimizer), and randomized frequencies (you
-    provide the number of frequencies, and the frequency values are chosen at
-    random at the start of each optimization and then held constant).
-
     Parameters
     ----------
     duration : float
         The total duration :math:`\tau` of the signal.
-    segments_count : int
+    segment_count : int
         The number of segments :math:`N` to use for the piecewise-constant
         approximation.
     initial_coefficient_lower_bound : float, optional
@@ -638,10 +657,10 @@ class RealFourierPwcSignal(Node):
         basis. Must be non-empty if provided. Must be specified in the inverse
         units of `duration` (for example if `duration` is in seconds, these
         values must be given in Hertz).
-    optimizable_frequencies_count : int, optional
+    optimizable_frequency_count : int, optional
         The number of non-zero frequencies :math:`M` to use, if the
         frequencies can be optimized. Must be greater than zero if provided.
-    randomized_frequencies_count : int, optional
+    randomized_frequency_count : int, optional
         The number of non-zero frequencies :math:`M` to use, if the
         frequencies are to be randomized but fixed. Must be greater than zero
         if provided.
@@ -650,10 +669,19 @@ class RealFourierPwcSignal(Node):
 
     Returns
     -------
-    TensorPwc(1D, real)
+    Pwc(1D, real)
         The optimizable, real-valued, piecewise-constant signal built from the
         appropriate Fourier components.
 
+    Warnings
+    --------
+    You must provide exactly one of `fixed_frequencies`, `optimizable_variable`,
+    or `randomized_frequency_count`.
+
+    See Also
+    --------
+    real_fourier_stf_signal : Corresponding operation for `Stf`.
+
     Notes
     -----
     This function sets the basis signal frequencies :math:`\{f_m\}`
@@ -696,50 +724,40 @@ class RealFourierPwcSignal(Node):
     optimizable_variable = True
     args = [
         forge.arg("duration", type=float),
-        forge.arg("segments_count", type=int),
+        forge.arg("segment_count", type=int),
         forge.arg("initial_coefficient_lower_bound", type=float, default=-1),
         forge.arg("initial_coefficient_upper_bound", type=float, default=1),
         forge.arg("fixed_frequencies", type=Optional[List[float]], default=None),
-        forge.arg("optimizable_frequencies_count", type=Optional[int], default=None),
-        forge.arg("randomized_frequencies_count", type=Optional[int], default=None),
+        forge.arg("optimizable_frequency_count", type=Optional[int], default=None),
+        forge.arg("randomized_frequency_count", type=Optional[int], default=None),
     ]
-    rtype = types.TensorPwc
+    rtype = Pwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         duration = kwargs.get("duration")
-        segments_count = kwargs.get("segments_count")
+        segment_count = kwargs.get("segment_count")
         check_duration(duration, "duration")
-        check_argument_integer(segments_count, "segments_count")
+
+        check_argument_integer(segment_count, "segment_count")
         check_argument(
-            segments_count > 0,
+            segment_count > 0,
             "The number of segments must be greater than zero.",
-            {"segments_count": segments_count},
+            {"segment_count": segment_count},
         )
-        durations = duration / segments_count * np.ones(segments_count)
-        optimizable_frequencies_count = kwargs.get("optimizable_frequencies_count")
-        if optimizable_frequencies_count is not None:
-            check_argument_integer(
-                optimizable_frequencies_count, "optimizable_frequencies_count"
-            )
-            check_argument(
-                optimizable_frequencies_count > 0,
-                "The number of optimizable frequencies (if provided) must be greater than zero.",
-                {"optimizable_frequencies_count": optimizable_frequencies_count},
-            )
-        randomized_frequencies_count = kwargs.get("randomized_frequencies_count")
-        if randomized_frequencies_count is not None:
-            check_argument_integer(
-                randomized_frequencies_count, "randomized_frequencies_count"
-            )
-            check_argument(
-                randomized_frequencies_count > 0,
-                "The number of randomized frequencies (if provided) must be greater than zero.",
-                {"randomized_frequencies_count": randomized_frequencies_count},
-            )
-        return TensorPwcNodeData(
+
+        fixed_frequencies = kwargs.get("fixed_frequencies")
+        optimizable_frequency_count = kwargs.get("optimizable_frequency_count")
+        randomized_frequency_count = kwargs.get("randomized_frequency_count")
+        validate_inputs_real_fourier_signal(
+            fixed_frequencies, optimizable_frequency_count, randomized_frequency_count
+        )
+
+        durations = duration / segment_count * np.ones(segment_count)
+
+        return Pwc(
             _operation,
-            values_shape=(),
+            value_shape=(),
             durations=durations,
             batch_shape=(),
         )
@@ -751,14 +769,14 @@ class SymmetrizePwc(Node):
 
     Parameters
     ----------
-    pwc : TensorPwc
+    pwc : Pwc
         The piecewise-constant function :math:`v(t)` to symmetrize.
     name : str, optional
         The name of the node.
 
     Returns
     -------
-    TensorPwc
+    Pwc
         The piecewise-constant function :math:`w(t)` defined by
         :math:`w(t)=v(t)` for :math:`0\leq t\leq \tau` and
         :math:`w(t)=v(2\tau-t)` for :math:`\tau\leq t\leq 2\tau`, where
@@ -766,29 +784,29 @@ class SymmetrizePwc(Node):
 
     See Also
     --------
-    pwc_signal : Create `TensorPwc` signals from (possibly complex) values.
+    pwc_signal : Create `Pwc` signals from (possibly complex) values.
     """
 
     name = "symmetrize_pwc"
-    args = [forge.arg("pwc", type=types.TensorPwc)]
-    rtype = types.TensorPwc
+    args = [forge.arg("pwc", type=Pwc)]
+    rtype = Pwc
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         pwc = kwargs.get("pwc")
         check_argument(
-            isinstance(pwc, TensorPwcNodeData),
-            "The pwc parameter must be a TensorPwc.",
+            isinstance(pwc, Pwc),
+            "The pwc parameter must be a Pwc.",
             {"pwc": pwc},
         )
-        batch_shape, values_shape = validate_batch_and_values_shapes(
+        batch_shape, value_shape = validate_batch_and_value_shapes(
             pwc,
             "pwc",
         )
         symmetrized_durations = np.concatenate((pwc.durations, pwc.durations[::-1]))
-        return TensorPwcNodeData(
+        return Pwc(
             _operation,
-            values_shape=values_shape,
+            value_shape=value_shape,
             durations=symmetrized_durations,
             batch_shape=batch_shape,
         )
@@ -801,7 +819,7 @@ class TimeEvolutionOperatorsPwc(Node):
 
     Parameters
     ----------
-    hamiltonian : TensorPwc
+    hamiltonian : Pwc
         The control Hamiltonian, or batch of control Hamiltonians.
     sample_times : np.ndarray(1D, real)
         The N times at which you want to sample the unitaries. Must be ordered and contain
@@ -824,26 +842,83 @@ class TimeEvolutionOperatorsPwc(Node):
 
     name = "time_evolution_operators_pwc"
     args = [
-        forge.arg("hamiltonian", type=types.TensorPwc),
+        forge.arg("hamiltonian", type=Pwc),
         forge.arg("sample_times", type=np.ndarray),
     ]
-    rtype = types.Tensor
+    rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         hamiltonian = kwargs.get("hamiltonian")
         sample_times = kwargs.get("sample_times")
         check_argument(
-            isinstance(hamiltonian, TensorPwcNodeData),
-            "The Hamiltonian must be a TensorPwc.",
+            isinstance(hamiltonian, Pwc),
+            "The Hamiltonian must be a Pwc.",
             {"hamiltonian": hamiltonian},
         )
-        batch_shape, values_shape = validate_batch_and_values_shapes(
+        batch_shape, value_shape = validate_batch_and_value_shapes(
             hamiltonian,
             "hamiltonian",
         )
         check_sample_times(sample_times, "sample_times")
         validate_hamiltonian(hamiltonian, "hamiltonian")
         time_count = len(sample_times)
-        shape = batch_shape + (time_count,) + values_shape
-        return TensorNodeData(_operation, shape=shape)
+        shape = batch_shape + (time_count,) + value_shape
+        return Tensor(_operation, shape=shape)
+
+
+class SamplePwc(Node):
+    r"""
+    Samples a Pwc at the given times.
+
+    Parameters
+    ----------
+    pwc : Pwc
+        The Pwc to sample.
+    sample_times : np.ndarray(1D, real)
+        The times at which you want to sample the Pwc. Must be ordered, contain
+        at least one element, and lie between 0 and the duration of the Pwc.
+        For a sample time :math:`t` the returned value
+        lies in the half open interval :math:`t_{j-1} < t \leq t_j`, where
+        :math:`t_j` indicates the boundary of a segment.
+    name : str, optional
+        The name of the node.
+
+    Returns
+    -------
+    Tensor
+        The values of the Pwc at the given times.
+
+    See Also
+    --------
+    sample_stf : Sample an `Stf` at given times.
+    """
+
+    name = "sample_pwc"
+    args = [
+        forge.arg("pwc", type=Pwc),
+        forge.arg("sample_times", type=np.ndarray),
+    ]
+    rtype = Tensor
+
+    @classmethod
+    def create_node_data(cls, _operation, **kwargs):
+        pwc = kwargs.get("pwc")
+        check_argument(isinstance(pwc, Pwc), "The pwc must be a Pwc.", {"pwc": pwc})
+        batch_shape, value_shape = validate_batch_and_value_shapes(pwc, "pwc")
+        sample_times = kwargs.get("sample_times")
+        check_sample_times(sample_times, "sample_times")
+        check_argument(
+            sample_times[0] >= 0,
+            "The first sample time can't be smaller than zero.",
+            {"sample_times": sample_times},
+        )
+        check_argument(
+            sample_times[-1] <= np.sum(pwc.durations),
+            "The last sample time can't be larger than the duration of the Pwc.",
+            {"sample_times": sample_times},
+            extras={"Pwc durations": pwc.durations},
+        )
+        time_count = len(sample_times)
+        shape = batch_shape + (time_count,) + value_shape
+        return Tensor(_operation, shape=shape)
