@@ -208,13 +208,13 @@ class StateEvolutionPwc(Node):
 
     Parameters
     ----------
-    initial_state : Union[Tensor, np.ndarray]
+    initial_state : Tensor or np.ndarray
         The initial state as a Tensor or np.ndarray of shape ``[D]``.
-    hamiltonian : Union[Pwc, SparsePwc]
+    hamiltonian : Pwc or SparsePwc
         The control Hamiltonian. Uses sparse matrix multiplication if of type
         `SparsePwc`, which can be more efficient for large operators that are
         relatively sparse (contain mostly zeros).
-    krylov_subspace_dimension : Union[Tensor, int]
+    krylov_subspace_dimension : Tensor or int
         The dimension of the Krylov subspace `k` for the Lanczos method.
     sample_times :  np.ndarray(1D, real), optional
         The N times at which you want to sample the state. Elements must be non-negative
@@ -327,7 +327,7 @@ class EstimatedKrylovSubspaceDimensionLanczos(Node):
 
     Parameters
     ----------
-    spectral_range : Union[Tensor, float]
+    spectral_range : Tensor or float
         Estimated order of magnitude of Hamiltonian spectral range (difference
         between largest and smallest eigenvalues).
     duration : float
