@@ -17,6 +17,7 @@
 from typing import (
     List,
     Optional,
+    Tuple,
     Union,
 )
 
@@ -39,15 +40,26 @@ class Sum(Node):
 
     Parameters
     ----------
-    input_tensor : np.ndarray or Tensor or List[Tensor]
-        The tensor whose elements you want to sum.
+    input_tensor : np.ndarray or Tensor or list[Tensor]
+        The tensor whose elements you want to sum. If you pass a list of tensors, they must all have
+        the same shape, and are interpreted as being stacked along a new first dimension (for
+        example, if you pass two 2D tensors of shape ``[3, 4]``, the result is equivalent to passing
+        the stacked 3D tensor of shape ``[2, 3, 4]``).
+    axis : int or list[int] or tuple[int], optional
+        The dimension or dimensions along which you want to sum the tensor. Defaults to `None`, in
+        which case this node sums along all axes of the tensor.
+    keepdims : bool, optional
+        Whether or not to retain summed axes in the output tensor. If true, each dimension in
+        `axis` has size 1 in the result; otherwise, the dimensions in `axis` are removed from the
+        result. Defaults to false.
     name : str, optional
         The name of the node.
 
     Returns
     -------
     Tensor
-        The single-element tensor containing the input tensor's sum.
+        The tensor obtained by summing the input tensor along the specified axes (or, if `axis` was
+        `None`, the tensor obtained by summing the input tensor along all of the specified axes).
     """
 
     name = "sum"
@@ -55,13 +67,27 @@ class Sum(Node):
         forge.arg(
             "input_tensor",
             type=Union[np.ndarray, Tensor, List[Tensor]],
-        )
+        ),
+        forge.arg(
+            "axis",
+            type=Optional[Union[List[int], Tuple[int, ...]]],
+            default=None,
+        ),
+        forge.arg(
+            "keepdims",
+            type=bool,
+            default=False,
+        ),
     ]
     rtype = Tensor
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         input_tensor = kwargs.get("input_tensor")
+        # Make a copy of the input, since we'll mutate it.
+        axis = kwargs.get("axis")
+        keepdims = kwargs.get("keepdims")
+
         check_argument_numeric(input_tensor, "input_tensor")
         if isinstance(input_tensor, list):
             shapes = [
@@ -78,7 +104,47 @@ class Sum(Node):
                         f"input_tensor[{index}].shape": shape,
                     },
                 )
-        return Tensor(_operation, shape=())
+            # Note that if the input is an empty list then the shape is somewhat ambiguous (it
+            # could be an empty list of tensors of any shape), but for consistency with TF and NP
+            # we interpret it as 1D.
+            shape = (len(shapes), *shapes[0]) if shapes else ()
+        else:
+            shape = validate_shape(input_tensor, "input_tensor")
+
+        # Validate and sanitize the reduction axes.
+        if axis is None:
+            axis = list(range(len(shape)))
+        elif isinstance(axis, int):
+            axis = [axis]
+        else:
+            axis = list(axis)
+        for i, dimension in enumerate(axis):
+            check_argument(
+                -len(shape) <= dimension < len(shape),
+                f"Elements of axis must be valid axes of the input_tensor (between {-len(shape)} "
+                f"and {len(shape)-1}, inclusive).",
+                {"input_tensor": input_tensor, "axis": axis},
+            )
+            if dimension < 0:
+                axis[i] = dimension + len(shape)
+        axis_set = set(axis)
+        check_argument(
+            len(axis_set) == len(axis),
+            "Elements of axis must refer to unique dimensions of the input_tensor.",
+            {"input_tensor": input_tensor, "axis": axis},
+        )
+
+        # Calculate the output shape.
+        output_shape = []
+        for i, size in enumerate(shape):
+            if i not in axis_set:
+                output_shape.append(size)
+                continue
+            if keepdims:
+                output_shape.append(1)
+                continue
+
+        return Tensor(_operation, shape=tuple(output_shape))
 
 
 class Reverse(Node):
@@ -89,7 +155,7 @@ class Reverse(Node):
     ----------
     tensor : np.ndarray or Tensor
         The tensor that you want to reverse.
-    axis : List[int]
+    axis : list[int]
         The dimensions along which you want to reverse the tensor.
     name : str, optional
         The name of the node.
@@ -123,7 +189,7 @@ class Repeat(Node):
     ----------
     input : np.ndarray or Tensor
         The tensor whose elements you want to repeat.
-    repeats : int or List[int]
+    repeats : int or list[int]
         The number of times to repeat each element. If you pass a single int or singleton list, that
         number of repetitions is applied to each element. Otherwise, you must pass a list with the
         same length as `input` along the specified `axis` (or the same total length as `input` if
@@ -233,7 +299,7 @@ class Transpose(Node):
     ----------
     a : np.ndarray or Tensor
         The tensor whose dimensions you want to permute, :math:`x`.
-    perm : List[int] or np.ndarray(int), optional
+    perm : list[int] or np.ndarray(int), optional
         The order of the input dimensions for the returned tensor. If you provide it, it must
         be a permutation of all integers between 0 and ``N-1``, where `N` is the rank of `a`.
         If you don't provide it, the order of the dimensions is inverted, that is to say,
