@@ -1,11 +1,11 @@
-# Copyright 2020 Q-CTRL Pty Ltd & Q-CTRL Inc. All rights reserved.
+# Copyright 2021 Q-CTRL. All rights reserved.
 #
 # Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
 # copying or use of this file, via any medium, is strictly prohibited.
 # Proprietary and confidential. You may not use this file except in compliance
 # with the License. You may obtain a copy of the License at
 #
-#     https://q-ctrl.com/terms
+#      https://q-ctrl.com/terms
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS. See the
@@ -14,7 +14,6 @@
 # pylint: disable=too-many-lines
 """Module for unary operation nodes."""
 
-import warnings
 from typing import (
     Tuple,
     Union,
@@ -23,14 +22,15 @@ from typing import (
 import forge
 import numpy as np
 
-from qctrlcommons.node import (
-    node_data,
-    types,
-)
 from qctrlcommons.node.base import Node
+from qctrlcommons.node.node_data import (
+    Pwc,
+    Stf,
+    Tensor,
+)
 from qctrlcommons.node.utils import (
     NumericOrFunction,
-    validate_batch_and_values_shapes,
+    validate_batch_and_value_shapes,
     validate_shape,
 )
 from qctrlcommons.preconditions import (
@@ -38,10 +38,10 @@ from qctrlcommons.preconditions import (
     check_argument_numeric,
 )
 
-MatrixOrFunction = Union[np.ndarray, types.Tensor, types.TensorPwc, types.Stf]
+MatrixOrFunction = Union[np.ndarray, Tensor, Pwc, Stf]
 
 
-def _create_flexible_unary_node_data(_operation, x, name, values_shape_changer=None):
+def _create_flexible_unary_node_data(_operation, x, name, value_shape_changer=None):
     """
     Common implementation of `create_node_data` for nodes acting on Tensors, Pwcs, and Stfs
     implementing unary functions.
@@ -50,69 +50,69 @@ def _create_flexible_unary_node_data(_operation, x, name, values_shape_changer=N
     ----------
     _operation : Operation
         The operation to implement.
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object on which the operation acts.
     name : str
         The name of the node.
-    values_shape_changer : Callable[[tuple], tuple], optional
+    value_shape_changer : Callable[[tuple], tuple], optional
         Callable that transforms the original shape of the object into the shape after the operation
         is applied. Defaults to an identity operation, that is to say, to not change the shape.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The operation acting on the object.
     """
 
     # By default don't change shapes.
-    if values_shape_changer is None:
-        values_shape_changer = lambda shape: shape
+    if value_shape_changer is None:
+        value_shape_changer = lambda shape: shape
 
-    if isinstance(x, node_data.StfNodeData):
+    if isinstance(x, Stf):
         check_argument(
             name is None,
             "You can't assign a name to an Stf node.",
             {"name": name},
         )
-        batch_shape, values_shape = validate_batch_and_values_shapes(x, "x")
-        return node_data.StfNodeData(
+        batch_shape, value_shape = validate_batch_and_value_shapes(x, "x")
+        return Stf(
             _operation,
-            values_shape=values_shape_changer(values_shape),
+            value_shape=value_shape_changer(value_shape),
             batch_shape=batch_shape,
         )
 
-    if isinstance(x, node_data.TensorPwcNodeData):
-        batch_shape, values_shape = validate_batch_and_values_shapes(x, "x")
-        return node_data.TensorPwcNodeData(
+    if isinstance(x, Pwc):
+        batch_shape, value_shape = validate_batch_and_value_shapes(x, "x")
+        return Pwc(
             _operation,
-            values_shape=values_shape_changer(values_shape),
+            value_shape=value_shape_changer(value_shape),
             durations=x.durations,
             batch_shape=batch_shape,
         )
 
     check_argument_numeric(x, "x")
     shape = validate_shape(x, "x")
-    return node_data.TensorNodeData(_operation, shape=values_shape_changer(shape))
+    return Tensor(_operation, shape=value_shape_changer(shape))
 
 
 class Sqrt(Node):
     r"""
     Returns the element-wise square root of an object. This can be a number, an array, a tensor, or
-    a time-dependent function in the form of a TensorPwc or an Stf.
+    a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose square root you want to calculate, :math:`x`. For numbers, arrays, and
         tensors, the object is converted to a tensor and then the operation is applied. For
-        functions of time (TensorPwcs and Stfs), the composition of the operation with the function
+        functions of time (Pwcs and Stfs), the composition of the operation with the function
         is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise square root, :math:`\sqrt{x}`, of the values or function you provided.
         The returned object is of the same kind as the one you provided, except if you provide a
         number or an np.ndarray in which case it's a Tensor.
@@ -120,7 +120,7 @@ class Sqrt(Node):
 
     name = "sqrt"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -130,21 +130,21 @@ class Sqrt(Node):
 class Sin(Node):
     r"""
     Returns the element-wise sine of an object. This can be a number, an array, a tensor, or
-    a time-dependent function in the form of a TensorPwc or an Stf.
+    a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose sine you want to calculate, :math:`x`. For numbers, arrays, and
         tensors, the object is converted to a tensor and then the operation is applied. For
-        functions of time (TensorPwcs and Stfs), the composition of the operation with the function
+        functions of time (Pwcs and Stfs), the composition of the operation with the function
         is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise sine, :math:`\sin{x}`, of the values or function you provided.
         The returned object is of the same kind as the one you provided, except if you provide a
         number or an np.ndarray in which case it's a Tensor.
@@ -152,7 +152,7 @@ class Sin(Node):
 
     name = "sin"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -162,21 +162,21 @@ class Sin(Node):
 class Cos(Node):
     r"""
     Returns the element-wise cosine of an object. This can be a number, an array, a tensor, or
-    a time-dependent function in the form of a TensorPwc or an Stf.
+    a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose cosine you want to calculate, :math:`x`. For numbers, arrays, and
         tensors, the object is converted to a tensor and then the operation is applied. For
-        functions of time (TensorPwcs and Stfs), the composition of the operation with the function
+        functions of time (Pwcs and Stfs), the composition of the operation with the function
         is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise cosine, :math:`\cos{x}`, of the values or function you provided.
         The returned object is of the same kind as the one you provided, except if you provide a
         number or an np.ndarray in which case it's a Tensor.
@@ -184,7 +184,7 @@ class Cos(Node):
 
     name = "cos"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -194,21 +194,21 @@ class Cos(Node):
 class Tan(Node):
     r"""
     Returns the element-wise tangent of an object. This can be a number, an array, a tensor, or
-    a time-dependent function in the form of a TensorPwc or an Stf.
+    a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose tangent you want to calculate, :math:`x`. For numbers, arrays, and
         tensors, the object is converted to a tensor and then the operation is applied. For
-        functions of time (TensorPwcs and Stfs), the composition of the operation with the function
+        functions of time (Pwcs and Stfs), the composition of the operation with the function
         is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise tangent, :math:`\tan{x}`, of the values or function you provided.
         The returned object is of the same kind as the one you provided, except if you provide a
         number or an np.ndarray in which case it's a Tensor.
@@ -216,7 +216,7 @@ class Tan(Node):
 
     name = "tan"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -226,21 +226,21 @@ class Tan(Node):
 class Sinh(Node):
     r"""
     Returns the element-wise hyperbolic sine of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose hyperbolic sine you want to calculate, :math:`x`. For numbers, arrays, and
         tensors, the object is converted to a tensor and then the operation is applied. For
-        functions of time (TensorPwcs and Stfs), the composition of the operation with the function
+        functions of time (Pwcs and Stfs), the composition of the operation with the function
         is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise hyperbolic sine, :math:`\sinh{x}`, of the values or function you provided.
         The returned object is of the same kind as the one you provided, except if you provide a
         number or an np.ndarray in which case it's a Tensor.
@@ -248,7 +248,7 @@ class Sinh(Node):
 
     name = "sinh"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -258,21 +258,21 @@ class Sinh(Node):
 class Cosh(Node):
     r"""
     Returns the element-wise hyperbolic cosine of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose hyperbolic cosine you want to calculate, :math:`x`. For numbers, arrays,
         and tensors, the object is converted to a tensor and then the operation is applied. For
-        functions of time (TensorPwcs and Stfs), the composition of the operation with the function
+        functions of time (Pwcs and Stfs), the composition of the operation with the function
         is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise hyperbolic cosine, :math:`\cosh{x}`, of the values or function you
         provided. The returned object is of the same kind as the one you provided, except if you
         provide a number or an np.ndarray in which case it's a Tensor.
@@ -280,7 +280,7 @@ class Cosh(Node):
 
     name = "cosh"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -290,21 +290,21 @@ class Cosh(Node):
 class Tanh(Node):
     r"""
     Returns the element-wise hyperbolic tangent of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose hyperbolic tangent you want to calculate, :math:`x`. For numbers, arrays,
         and tensors, the object is converted to a tensor and then the operation is applied. For
-        functions of time (TensorPwcs and Stfs), the composition of the operation with the function
+        functions of time (Pwcs and Stfs), the composition of the operation with the function
         is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise hyperbolic tangent, :math:`\tanh{x}`, of the values or function you
         provided. The returned object is of the same kind as the one you provided, except if you
         provide a number or an np.ndarray in which case it's a Tensor.
@@ -312,7 +312,7 @@ class Tanh(Node):
 
     name = "tanh"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -322,21 +322,21 @@ class Tanh(Node):
 class Log(Node):
     r"""
     Returns the element-wise natural logarithm of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose natural logarithm you want to calculate, :math:`x`. For numbers, arrays,
         and tensors, the object is converted to a tensor and then the operation is applied. For
-        functions of time (TensorPwcs and Stfs), the composition of the operation with the function
+        functions of time (Pwcs and Stfs), the composition of the operation with the function
         is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise natural logarithm, :math:`\log{x}`, of the values or function you
         provided. The returned object is of the same kind as the one you provided, except if you
         provide a number or an np.ndarray in which case it's a Tensor.
@@ -344,7 +344,7 @@ class Log(Node):
 
     name = "log"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -354,21 +354,21 @@ class Log(Node):
 class Exp(Node):
     r"""
     Returns the element-wise exponential of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose exponential you want to calculate, :math:`x`. For numbers, arrays,
         and tensors, the object is converted to a tensor and then the operation is applied. For
-        functions of time (TensorPwcs and Stfs), the composition of the operation with the function
+        functions of time (Pwcs and Stfs), the composition of the operation with the function
         is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise exponential, :math:`e^{x}`, of the values or function you
         provided. The returned object is of the same kind as the one you provided, except if you
         provide a number or an np.ndarray in which case it's a Tensor.
@@ -376,7 +376,7 @@ class Exp(Node):
 
     name = "exp"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -386,21 +386,21 @@ class Exp(Node):
 class Negative(Node):
     r"""
     Returns the element-wise numerical negative value of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose numerical negative value you want to calculate, :math:`x`. For numbers,
         arrays, and tensors, the object is converted to a tensor and then the operation is
-        applied. For functions of time (TensorPwcs and Stfs), the composition of the operation with
+        applied. For functions of time (Pwcs and Stfs), the composition of the operation with
         the function is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise negation, :math:`-x`, of the values or function you
         provided. The returned object is of the same kind as the one you provided, except if you
         provide a number or an np.ndarray in which case it's a Tensor.
@@ -408,47 +408,31 @@ class Negative(Node):
 
     name = "negative"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
         return _create_flexible_unary_node_data(_operation, **kwargs)
 
 
-class NegDeprecated(Negative):
-    r"""
-    This node has been deprecated and will be removed in the future.
-    Please use :py:func:`negative` instead.
-    """
-    name = "neg"
-
-    @classmethod
-    def create_node_data(cls, _operation, **kwargs):
-        warnings.warn(
-            "The 'neg' node will be removed in the future. "
-            "Please use 'negative' instead."
-        )
-        return super().create_node_data(_operation, **kwargs)
-
-
 class Real(Node):
     r"""
     Returns the element-wise real part of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose real part you want to calculate, :math:`x`. For numbers,
         arrays, and tensors, the object is converted to a tensor and then the operation is
-        applied. For functions of time (TensorPwcs and Stfs), the composition of the operation with
+        applied. For functions of time (Pwcs and Stfs), the composition of the operation with
         the function is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise real part, :math:`\Re(x)`, of the values or function you provided. The
         returned object is a real object of the same kind as the one you provided, except if you
         provide a number or an np.ndarray in which case it's a Tensor.
@@ -461,7 +445,7 @@ class Real(Node):
 
     name = "real"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -471,21 +455,21 @@ class Real(Node):
 class Imag(Node):
     r"""
     Returns the element-wise imaginary part of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose imaginary part you want to calculate, :math:`x`. For numbers,
         arrays, and tensors, the object is converted to a tensor and then the operation is
-        applied. For functions of time (TensorPwcs and Stfs), the composition of the operation with
+        applied. For functions of time (Pwcs and Stfs), the composition of the operation with
         the function is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise imaginary part, :math:`\Im(x)`, of the values or function you provided. The
         returned object is a real object of the same kind as the one you provided, except if you
         provide a number or an np.ndarray in which case it's a Tensor.
@@ -498,7 +482,7 @@ class Imag(Node):
 
     name = "imag"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -508,21 +492,21 @@ class Imag(Node):
 class Absolute(Node):
     r"""
     Returns the element-wise absolute value of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose absolute value you want to calculate, :math:`x`. For numbers,
         arrays, and tensors, the object is converted to a tensor and then the operation is
-        applied. For functions of time (TensorPwcs and Stfs), the composition of the operation with
+        applied. For functions of time (Pwcs and Stfs), the composition of the operation with
         the function is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise absolute value, :math:`\left|x\right|`, of the values or function you
         provided. The returned object is a real object of the same kind as the one you provided,
         except if you provide a number or an np.ndarray in which case it's a Tensor.
@@ -535,7 +519,7 @@ class Absolute(Node):
 
     name = "abs"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -545,21 +529,21 @@ class Absolute(Node):
 class Angle(Node):
     r"""
     Returns the element-wise argument of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose argument you want to calculate, :math:`x`. For numbers,
         arrays, and tensors, the object is converted to a tensor and then the operation is
-        applied. For functions of time (TensorPwcs and Stfs), the composition of the operation with
+        applied. For functions of time (Pwcs and Stfs), the composition of the operation with
         the function is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise argument, :math:`\arg(x)`, of the values or function you provided. The
         returned object is a real object of the same kind as the one you provided, except if you
         provide a number or an np.ndarray in which case it's a Tensor.
@@ -572,7 +556,7 @@ class Angle(Node):
 
     name = "angle"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -582,21 +566,21 @@ class Angle(Node):
 class Conjugate(Node):
     r"""
     Returns the element-wise complex conjugate of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : number or np.ndarray or Tensor or TensorPwc or Stf
+    x : number or np.ndarray or Tensor or Pwc or Stf
         The object whose complex conjugate you want to calculate, :math:`x`. For numbers,
         arrays, and tensors, the object is converted to a tensor and then the operation is
-        applied. For functions of time (TensorPwcs and Stfs), the composition of the operation with
+        applied. For functions of time (Pwcs and Stfs), the composition of the operation with
         the function is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise complex conjugate, :math:`x^\ast`, of the values or function you
         provided. The returned object is of the same kind as the one you provided, except if you
         provide a number or an np.ndarray in which case it's a Tensor.
@@ -608,7 +592,7 @@ class Conjugate(Node):
 
     name = "conjugate"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -618,21 +602,21 @@ class Conjugate(Node):
 class Arcsin(Node):
     r"""
     Returns the element-wise arcsine of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : float or np.ndarray or Tensor or TensorPwc or Stf
+    x : float or np.ndarray or Tensor or Pwc or Stf
         The object whose arcsine you want to calculate, :math:`x`. Must be real. For numbers,
         arrays, and tensors, the object is converted to a tensor and then the operation is applied.
-        For functions of time (TensorPwcs and Stfs), the composition of the operation with the
+        For functions of time (Pwcs and Stfs), the composition of the operation with the
         function is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise arcsine, :math:`\arcsin{x}`, of the values or function you
         provided. Outputs will be in the range of :math:`[-\pi/2, \pi/2]`.
         The returned object is of the same kind as the one you provided, except if you
@@ -641,7 +625,7 @@ class Arcsin(Node):
 
     name = "arcsin"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -651,21 +635,21 @@ class Arcsin(Node):
 class Arccos(Node):
     r"""
     Returns the element-wise arccosine of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : float or np.ndarray or Tensor or TensorPwc or Stf
+    x : float or np.ndarray or Tensor or Pwc or Stf
         The object whose arccosine you want to calculate, :math:`x`. Must be real. For numbers,
         arrays, and tensors, the object is converted to a tensor and then the operation is applied.
-        For functions of time (TensorPwcs and Stfs), the composition of the operation with the
+        For functions of time (Pwcs and Stfs), the composition of the operation with the
         function is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise arccosine, :math:`\arccos{x}`, of the values or function you
         provided. Outputs will be in the range of :math:`[0, \pi]`.
         The returned object is of the same kind as the one you provided, except if you
@@ -674,7 +658,7 @@ class Arccos(Node):
 
     name = "arccos"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -684,21 +668,21 @@ class Arccos(Node):
 class Arctan(Node):
     r"""
     Returns the element-wise arctangent of an object. This can be a number, an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf.
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf.
 
     Parameters
     ----------
-    x : float or np.ndarray or Tensor or TensorPwc or Stf
+    x : float or np.ndarray or Tensor or Pwc or Stf
         The object whose arctangent you want to calculate, :math:`x`. Must be real. For numbers,
         arrays, and tensors, the object is converted to a tensor and then the operation is applied.
-        For functions of time (TensorPwcs and Stfs), the composition of the operation with the
+        For functions of time (Pwcs and Stfs), the composition of the operation with the
         function is computed (that is, the operation is applied to the function values).
     name : str, optional
         The name of the node. You can only provide a name if the object is not an Stf.
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise arctangent, :math:`\arctan{x}`, of the values or function you
         provided. Outputs will be in the range of :math:`[-\pi/2, \pi/2]`.
         The returned object is of the same kind as the one you provided, except if you
@@ -707,7 +691,7 @@ class Arctan(Node):
 
     name = "arctan"
     args = [forge.arg("x", type=NumericOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
@@ -718,15 +702,15 @@ class Adjoint(Node):
     r"""
     Returns the element-wise adjoint of the last two dimensions of an object.
     This can be a an array, a tensor, or a time-dependent function in the form of a
-    TensorPwc or an Stf where values have at least two dimensions.
+    Pwc or an Stf where values have at least two dimensions.
 
     Parameters
     ----------
-    x : np.ndarray or Tensor or TensorPwc or Stf
+    x : np.ndarray or Tensor or Pwc or Stf
         The object whose adjoint you want to calculate, :math:`X^\dagger`.
         Must be a matrix or a matrix-valued function.
         For arrays and tensors, the object is converted to a tensor and then
-        the operation is applied. For functions of time (TensorPwcs and Stfs), the composition
+        the operation is applied. For functions of time (Pwcs and Stfs), the composition
         of the operation with the function is computed
         (that is, the operation is applied to the function values).
     name : str, optional
@@ -734,7 +718,7 @@ class Adjoint(Node):
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise adjoint, of the last two dimension of the given matrix or matrix-valued
         function.
 
@@ -746,40 +730,40 @@ class Adjoint(Node):
 
     name = "adjoint"
     args = [forge.arg("x", type=MatrixOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
-        def values_shape_changer(values_shape: Tuple[int]) -> Tuple[int]:
+        def value_shape_changer(value_shape: Tuple[int]) -> Tuple[int]:
             check_argument(
-                len(values_shape) >= 2,
+                len(value_shape) >= 2,
                 "x must be at least 2D.",
                 {"x": kwargs.get("x")},
-                extras={"values_shape(x)": values_shape},
+                extras={"x.value_shape": value_shape},
             )
             # Unpacking syntax is awesome.
-            *batch, x, y = values_shape
+            *batch, x, y = value_shape
             return (*batch, y, x)
 
         return _create_flexible_unary_node_data(
-            _operation, **kwargs, values_shape_changer=values_shape_changer
+            _operation, **kwargs, value_shape_changer=value_shape_changer
         )
 
 
 class Trace(Node):
     r"""
     Returns the element-wise trace of an object. This can be a an array,
-    a tensor, or a time-dependent function in the form of a TensorPwc or an Stf
+    a tensor, or a time-dependent function in the form of a Pwc or an Stf
     where values have at least two dimensions.
     The trace is calculated on the last two dimensions.
 
     Parameters
     ----------
-    x : np.ndarray or Tensor or TensorPwc or Stf
+    x : np.ndarray or Tensor or Pwc or Stf
         The object whose trace you want to calculate, :math:`\mathop{\mathrm{Tr}}(x)`.
         Must be a matrix or a matrix-valued function.
         For arrays and tensors, the object is converted to a tensor and then
-        the operation is applied. For functions of time (TensorPwcs and Stfs), the composition
+        the operation is applied. For functions of time (Pwcs and Stfs), the composition
         of the operation with the function is computed
         (that is, the operation is applied to the function values).
     name : str, optional
@@ -787,26 +771,26 @@ class Trace(Node):
 
     Returns
     -------
-    Tensor or TensorPwc or Stf
+    Tensor or Pwc or Stf
         The element-wise trace, of the last two dimension of the given matrix or matrix-valued
         function. Outputs will have two fewer dimensions.
     """
 
     name = "trace"
     args = [forge.arg("x", type=MatrixOrFunction)]
-    rtype = Union[types.Tensor, types.TensorPwc, types.Stf]
+    rtype = Union[Tensor, Pwc, Stf]
 
     @classmethod
     def create_node_data(cls, _operation, **kwargs):
-        def values_shape_changer(values_shape: Tuple[int]) -> Tuple[int]:
+        def value_shape_changer(value_shape: Tuple[int]) -> Tuple[int]:
             check_argument(
-                len(values_shape) >= 2,
+                len(value_shape) >= 2,
                 "x must be at least 2D.",
                 {"x": kwargs.get("x")},
-                extras={"values_shape(x)": values_shape},
+                extras={"x.value_shape": value_shape},
             )
-            return values_shape[:-2]
+            return value_shape[:-2]
 
         return _create_flexible_unary_node_data(
-            _operation, **kwargs, values_shape_changer=values_shape_changer
+            _operation, **kwargs, value_shape_changer=value_shape_changer
         )
